library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- ------------------------------------------------------------------
-- Module:    ClkGen
-- Purpose:   derive multiple clocks from system clock
-- Author:    myself
-- Created:   20090816
-- Behaviour: a system clock of 200 MHz is divided into 100, 50, 12.5 
--            and ~1.5 MHz
-- ------------------------------------------------------------------
ENTITY ClkGen IS
    Port    ( sysClk  : in  std_logic;
              reset   : in  std_logic;
              clk200  : out std_logic;
              clk100  : out std_logic;
              clk50   : out std_logic;
              clk12_5 : out std_logic;
              clk1_5  : out std_logic
            );
END ClkGen;

ARCHITECTURE std of ClkGen IS
   type reg_type is record
      cnt     : integer range 0 to 134*8-1;
      clk100  : std_logic;    -- divider 2
      clk50   : std_logic;    -- divider 4
      clk12_5 : std_logic;    -- divider 16
      clk1_5  : std_logic;    -- divider 134
   end record;
   constant CNT_MAX : integer := 134*8-1;
   constant rdef : reg_type := (
      cnt     => 0,
      clk100  => '0',
      clk50   => '0',
      clk12_5 => '0',
      clk1_5  => '0' 
   );      
   signal r, rin : reg_type; 
BEGIN
   clk200 <= sysClk;


   comp: PROCESS(r)
      variable v : reg_type;
   BEGIN
      v.clk100  := '0';
      v.clk50   := '0';
      v.clk12_5 := '0';
      v.clk1_5  := '0';
   
      if (r.cnt < CNT_MAX) then  -- next counter step
         v.cnt := r.cnt + 1;
      else
         v.cnt := 0;
      end if;

      if (v.cnt mod 2 = 0) then       -- calc derived clocks
         v.clk100 := '1';
      end if;
      if (v.cnt mod 4 = 0) then
         v.clk50 := '1';
      end if;
      if (v.cnt mod 16 = 0) then
         v.clk12_5 := '1';
      end if;   
--      case v.cnt is
--      when 0 | 134 | 268 | 402 | 536 | 670 | 804 | 938 =>
      if (v.cnt mod 134 = 0) then
         v.clk1_5 := '1';
      end if;   
--      when others => NULL;
--      end case;
      clk100  <= r.clk100;
      clk50   <= r.clk50;
      clk12_5 <= r.clk12_5;
      clk1_5  <= r.clk1_5;
      rin     <= v;
   END PROCESS;

   
   regs: PROCESS(sysClk, reset)
   BEGIN
      if    reset = '1'         then r <= rdef;
      elsif rising_edge(sysClk) then r <= rin;
      end if;
   END PROCESS;
END std;
