library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- ------------------------------------------------------------------
-- Module:    ClkGen
-- Purpose:   derive multiple clocks from system clock
-- Author:    myself
-- Created:   20090816
-- Behaviour: a system clock of 200 MHz is divided into 100, 50, 12.5 
--            and ~1.5 MHz
-- ------------------------------------------------------------------
ENTITY ClkGen IS
    Port    ( sysClk  : in  std_logic;
              reset   : in  std_logic;
              clk200  : out std_logic;
              clk100  : out std_logic;
              clk50   : out std_logic;
              clk12_5 : out std_logic;
              clk1_5  : out std_logic 
            );
END ClkGen;

ARCHITECTURE std of ClkGen IS
   constant MxCnt : integer := 134*8-1;
   signal ce100  : std_logic;
   signal ce50   : std_logic;
   signal ce12_5 : std_logic;
   signal ce1_5  : std_logic;
   signal cnt    : integer range 0 to MxCnt;
BEGIN
   clk200 <= sysClk;

   PROCESS(reset, sysClk)
      variable tmp : integer range 0 to MxCnt;
   BEGIN
      if reset = '1' then 
         cnt    <=  0;
         ce100  <= '0';
         ce50   <= '0';
         ce12_5 <= '0';
         ce1_5  <= '0';
      elsif rising_edge(sysClk) then
         if (cnt < MxCnt) then
            tmp := cnt + 1;
         else   
            tmp := 0;
         end if;
         
         ce100  <= '0';
         ce50   <= '0';
         ce12_5 <= '0';
         ce1_5  <= '0';

         if (tmp mod 2 = 0) then
            ce100 <= '1';
         end if;
         if (tmp mod 4 = 0) then
            ce50 <= '1';
         end if;
         if (tmp mod 16 = 0) then
            ce12_5 <= '1';
         end if;
         case tmp is
         when 0 | 134 | 268 | 402 | 536 | 670 | 804 | 938 =>
            ce1_5 <= '1';
         when others => NULL;
         end case;
         cnt <= tmp;   
      end if;
   END PROCESS;

   PROCESS BEGIN
      wait until rising_edge(sysClk);
      if (ce100  = '1') then clk100  <= sysClk; end if;
      if (ce50   = '1') then clk50   <= sysClk; end if;
      if (ce12_5 = '1') then clk12_5 <= sysClk; end if;
      if (ce1_5  = '1') then clk1_5  <= sysClk; end if;
   END PROCESS;
END std;
